@@ -1,27 +1,95 @@
 package App::termpub;
-use Mojo::Base 'App::termpub::Pager';
-use Mojo::Util 'decode';
+use Mojo::Base 'App::termpub::Pager::HTML';
+use Mojo::Util 'decode', 'getopt';
 use Mojo::URL;
-use Mojo::File 'tempfile';
+use Mojo::File 'tempfile',    'path';
 use Mojo::JSON 'encode_json', 'decode_json';
-use App::termpub::Renderer;
+use App::termpub::Hyphen;
+use App::termpub::Epub;
+use App::termpub::Pager::Text;
 use Curses;
 
-our $VERSION = '1.04';
+our $VERSION = '1.05';
 
-has 'epub';
+has epub => sub {
+    my $self = shift;
+    App::termpub::Epub->new( filename => $self->filename );
+};
 has chapters => sub { shift->epub->chapters };
 has chapter  => sub { shift->epub->start_chapter };
-has 'hrefs';
-has history => sub { [ shift->chapter ] };
+has history  => sub { [ shift->chapter ] };
 has history_index => 0;
-has 'renderer';
+
+has hyphenation => sub { shift->config->{hyphenation} };
+has language    => sub { shift->config->{language} };
+has width       => sub { shift->config->{width} };
+has 'filename';
+
+has config => sub {
+    my $self        = shift;
+    my $config_file = "$ENV{HOME}/.termpubrc";
+    my $config      = { hyphenation => 1, language => 'en-US', width => 80 };
+    if ( !-e $config_file ) {
+        my $dir = $ENV{XDG_CONFIG_HOME} // '~/.config/';
+        $config_file = "$dir/termpub/config";
+    }
+    if ( -e $config_file ) {
+        $self->read_config( path($config_file), $config );
+    }
+    return $config;
+};
+
+sub read_config {
+    my ( $self, $file, $config ) = @_;
+    my $fh = $file->open('<:encoding(UTF-8)');
+    while (<$fh>) {
+        s/#.*//;
+        next if /^\s*$/;
+        my ( $cmd, @args ) = split;
+        if ( $cmd eq 'set' ) {
+            my ( $key, $val ) = @args;
+            if ( not exists $config->{$key} ) {
+                die "Unknown variable $key in config file.\n";
+            }
+            $val //= 1;
+            if ( $val eq 'true' || $val eq 'on' ) {
+                $val = 1;
+            }
+            elsif ( $val eq 'false' || $val eq 'off' ) {
+                $val = 0;
+            }
+            $config->{$key} = $val;
+        }
+    }
+    return $config;
+}
+
+has hyphenator => sub {
+    my $self = shift;
+    return if !$self->hyphenation;
+    my $lang = $self->epub->language || $self->language;
+    my $h    = App::termpub::Hyphen->new( lang => $lang );
+    return if !$h->installed;
+    return $h;
+};
+
+sub parse_argv {
+    my ( $self, $argv ) = @_;
+    my $handler = sub { my ( $n, $v ) = @_; $self->$n($v) };
+    local $SIG{__WARN__} = sub { die @_ };
+    getopt( $argv, 'language|l=s' => $handler, 'hyphenation!' => $handler );
+    die "Missing filename for epub.\n" if !$argv->[0];
+    $self->filename( $argv->[0] );
+}
 
 sub run {
-    my $self = shift;
+    my ( $self, $argv ) = @_;
+
+    $self->parse_argv($argv);
+
+    $self->pad_columns( $self->width );
 
     $self->title( $self->chapters->[ $self->chapter ]->title );
-    $self->render_pad;
 
     my $data = $self->epub->read_metadata;
 
@@ -37,6 +105,7 @@ sub run {
     $self->key_bindings->{t}                  = 'jump_to_toc';
     $self->key_bindings->{'<'}                = 'history_back';
     $self->key_bindings->{'>'}                = 'history_forward';
+    $self->key_bindings->{'|'}                = 'set_width';
     $self->key_bindings->{Curses::KEY_RESIZE} = 'handle_resize';
 
     $self->SUPER::run;
@@ -69,10 +138,21 @@ sub get_position {
     return $position;
 }
 
+sub set_width {
+    my ( $self, $num ) = @_;
+    $num ||= $self->prefix;
+    if ($num) {
+        $self->pad_columns($num);
+        $self->render;
+        $self->update_screen;
+    }
+    return;
+}
+
 sub handle_resize {
     my $self = shift;
     $self->SUPER::handle_resize;
-    $self->render_pad;
+    $self->render;
     $self->update_screen;
 }
 
@@ -99,7 +179,7 @@ sub open_image {
 sub open_link {
     my $self = shift;
     if ( $self->prefix ) {
-        my ( $type, $href ) = @{ $self->hrefs->[ $self->prefix - 1 ] };
+        my ( $type, $href ) = @{ $self->hrefs->[ $self->prefix - 1 ] || [] };
         return if !$href;
 
         if ( $type eq 'img' ) {
@@ -129,7 +209,7 @@ sub open_link {
                 $self->set_chapter($i);
 
                 if ( my $fragment = $url->fragment ) {
-                    if ( my $line = $self->renderer->id_line->{$fragment} ) {
+                    if ( my $line = $self->id_line->{$fragment} ) {
                         $self->line($line);
                     }
                 }
@@ -143,27 +223,22 @@ sub open_link {
 
 sub help_screen {
     my $self = shift;
-    my $pad  = newpad( scalar keys %{ $self->key_bindings }, $self->columns );
     my @keys = sort keys %{ $self->key_bindings };
 
-    my $row    = 0;
     my $length = 0;
     for my $key (@keys) {
         my $str = $keycodes{$key} || $key;
         $length = length($str) if length($str) > $length;
     }
 
+    my $content;
     for my $key (@keys) {
-        $pad->addstring( ( $keycodes{$key} || $key ) );
-        $pad->addstring( $row, $length,
-            ' = ' . $self->key_bindings->{$key} . "\n" );
-        $row++;
+        next if $key eq '' . Curses::KEY_RESIZE;
+        $content .= sprintf "%-*s = %s\n", $length, $keycodes{$key} || $key,
+          $self->key_bindings->{$key};
     }
 
-    my $pager = App::termpub::Pager->new;
-    $pager->pad($pad);
-    $pager->run;
-
+    App::termpub::Pager::Text->new( content => $content )->render->run;
     $self->update_screen;
 }
 
@@ -180,7 +255,7 @@ sub set_chapter {
         $self->history_index(0);
     }
     $self->chapter($num);
-    $self->render_pad;
+    $self->render;
     $self->title( $self->chapters->[$num]->title );
     $self->set_mark;
     $self->line(0);
@@ -205,26 +280,6 @@ sub history_forward {
     return;
 }
 
-sub next_chapter {
-    my $self = shift;
-    while (1) {
-        if ( $self->chapters->[ $self->chapter + 1 ] ) {
-            $self->set_chapter( $self->chapter + 1 );
-            if ( $self->pad ) {
-                $self->update_screen;
-                return;
-            }
-            else {
-                next;
-            }
-        }
-        else {
-            return;
-        }
-    }
-    return;
-}
-
 sub next_page {
     my $self = shift;
     $self->next_chapter if !$self->SUPER::next_page;
@@ -235,16 +290,13 @@ sub prev_page {
     $self->prev_chapter if !$self->SUPER::prev_page;
 }
 
-sub prev_chapter {
+sub next_chapter {
     my $self = shift;
     while (1) {
-        if ( $self->chapter > 0 ) {
-            $self->set_chapter( $self->chapter - 1 );
-            if ( $self->pad ) {
-                $self->update_screen;
-                return 1;
-            }
-            next;
+        if ( $self->chapters->[ $self->chapter + 1 ] ) {
+            $self->set_chapter( $self->chapter + 1 );
+            $self->update_screen;
+            return 1;
         }
         else {
             return;
@@ -253,15 +305,25 @@ sub prev_chapter {
     return;
 }
 
-sub render_pad {
-    my $self     = shift;
-    my $content  = $self->chapters->[ $self->chapter ]->content;
-    my $renderer = App::termpub::Renderer->new;
-    $renderer->render( decode( 'UTF-8', $content ) );
-    $self->renderer($renderer);
-    $self->pad( $renderer->pad );
-    $self->hrefs( $renderer->hrefs );
+sub prev_chapter {
+    my $self = shift;
+    while (1) {
+        if ( $self->chapter > 0 ) {
+            $self->set_chapter( $self->chapter - 1 );
+            $self->update_screen;
+            return 1;
+        }
+        else {
+            return;
+        }
+    }
     return;
 }
 
+sub render {
+    my $self    = shift;
+    my $content = $self->chapters->[ $self->chapter ]->content;
+    return $self->SUPER::render( decode( 'UTF-8', $content ) );
+}
+
 1;
